<?php
if( !function_exists( 'http_build_query' ) )
{
	function http_build_query( $formdata, $numeric_prefix = null, $key = null )
	{
		$res = array();
		foreach( (array)$formdata as $k=>$v )
		{
			$tmp_key = urlencode( is_int($k) ? $numeric_prefix.$k : $k );
			if( $key ) 
				$tmp_key = $key.'['.$tmp_key.']';

			if ( is_array( $v ) || is_object( $v ) )
				$res[] = http_build_query( $v, null, $tmp_key );
			else
				$res[] = $tmp_key."=".urlencode( $v );
		}

		$separator = ini_get( 'arg_separator.output' );
		return implode( $separator, $res );
	}
}

if( !function_exists( 'file_put_contents' ) )
{
	function file_put_contents( $file, $string )
	{
		$f = fopen( $file, 'a+' );
		ftruncate( $f, 0 );
		fwrite( $f, $string );
		fclose( $f );
	}
}

if( !function_exists( 'normalize_url' ) )
{
    function normalize_url( $url )
    {
        $url = trim( $url );
        
        $parts = parse_url( $url );
        $scheme = isset( $parts['scheme'] ) ? $parts['scheme'] : null;

        if( !$scheme )
        {
            $url = 'http://' . $url;
            $parts = parse_url( $url );
        }

        $path = isset( $parts['path'] ) ? $parts['path'] : null;
        
        if( !$path )
            $url .= '/';
        
        return $url;
    }
}

if( !function_exists( 'sign_reply' ) )
{
/**
This function was taken from PHP-openid-0.0.8.3 library. 
http://videntity.org/openid/
**/
	function sign_reply( $reply, $key, $signed_fields )
	{
		$text = '';
        
		foreach( $signed_fields as $i )
		{
			if( isset( $reply['openid.' . $i] ) )
				$val = $reply['openid.' . $i];
			else if( isset( $reply['openid_' . $i] ) )
				$val = $reply['openid_' . $i];

			$text .= sprintf( "%s:%s\n", $i, $val );
		}

		$sha1 = oidUtilPhpSha1::hmacsha1( $key, $text );
		$b64_sha1 = base64_encode( $sha1 );
        
		return array( implode( ',', $signed_fields ), $b64_sha1 );
    }
}

class OpenIDConsumer
{
	function check_identity( $identity_url, $trust_root, $return_to, $data, $immediate )
	{
		$url_info = OpenIDConsumer::parse_identity_url( $identity_url );
		if( $url_info['openid_server'] == '' )
			die( 'Couldn\'t find OpenID Server<br/>' );
		
		$assoc_info = OpenIDConsumer::get_association_by_server( $url_info['openid_server'] );

		$data['identity_url'] = $identity_url;

		if( count( $data ) > 0 )
		{
			if( !is_dir( './tmp/' ) )
				mkdir( './tmp/', 0757 );
			do
			{
				$file_name = rand() . '.txt';
			}
			while( is_file( './tmp/' . $file_name ) );

			OpenIDConsumer::delete_old_files();

			@file_put_contents( './tmp/' . $file_name, serialize( $data ) );

			$sep = strchr( $return_to, '?' ) ? '&': '?';
			$rt = $return_to . $sep . 'file_name=' . $file_name;
		}

		if( $immediate )
			OpenIDConsumer::checkid_immediate( $url_info['openid_server'], $url_info['server_id'], $trust_root, $rt, $assoc_info['assoc_handle'] );
		else
			OpenIDConsumer::checkid_setup( $url_info['openid_server'], $url_info['server_id'], $trust_root, $rt, $assoc_info['assoc_handle'] );
	}

	function login( $query )
	{
		if( isset( $query['openid_user_setup_url'] ) && $query['openid_user_setup_url'] != '' )
		{
			header( 'Location: ' . $query['openid_user_setup_url'] );
			exit;
		}

		if( !isset( $query['openid_return_to'] ) )
			die( 'Couldn\'t login this this OpenID identity.' );

		if( isset( $query['openid_invalidate_handle'] ) )
		{
			$to_remove = OpenIDConsumer::get_association_by_handle( $query['openid_assoc_handle'] );
			if( $to_remove )
				OpenIDConsomer::remove_association( $to_remove['assoc_handle'] );
		}

		$assoc_info = OpenIDConsumer::get_association_by_handle( $query['openid_assoc_handle'] );
		if( !$assoc_info )
		{
			if( !OpenIDConsumer::check_authentication( $query ) )
				die( 'Couldn\'t check authentication.' );
		}

		$signed_fields = explode( ',', trim( $query['openid_signed'] ) );
		list( $signed, $sig ) = sign_reply( $query, $assoc_info['secret'], $signed_fields );

		if( $query['openid_sig'] != $sig )
			die( 'Couldn\'t verify sign.' );

		$parts = parse_url( $query['openid_return_to'] );
		$params_str = isset( $parts['query'] ) ? $parts['query']: null;
		parse_str( $params_str, $params );
		if( isset( $params['file_name'] ) )
		{
			$content = @file_get_contents( './tmp/' . $params['file_name'] );
			$data = $content ? unserialize( $content ): array();
			@unlink( './tmp/' . $params['file_name'] );
		}

		$identity_url = $query['openid_identity'];
		if( $data['identity_url'] != '' )
			$identity_url = $data['identity_url'];

		if( preg_match( '/^http:\/\/([0-9a-zA-Z-_]*)\.livejournal\.com\/?$/i', $identity_url, $matches ) )
			$openid_name = $matches[1] . "@livejournal";
		elseif( preg_match( '/^http:\/\/(www\.)?livejournal\.com\/users\/([0-9a-zA-Z-_]+)\/?$/i', $identity_url, $matches ) )
			$openid_name = $matches[2] . "@livejournal";
		elseif( preg_match( '/^http:\/\/(www\.)?livejournal\.com\/~([0-9a-zA-Z-_]+)\/?$/i', $identity_url, $matches ) )
			$openid_name = $matches[2] . "@livejournal";
		elseif( preg_match( '/^http:\/\/([0-9a-zA-Z-_]+)\.deadjournal\.com\/?$/i', $identity_url, $matches ) )
			$openid_name = $matches[1] . "@deadjournal";
		elseif( preg_match( '/^http:\/\/(www\.)?deadjournal\.com\/users\/([0-9a-zA-Z-_]+)\/?$/i', $identity_url, $matches ) )
			$openid_name = $matches[2] . "deadjournal";
		elseif( preg_match( '/^http:\/\/(www\.)?deadjournal\.com\/~([0-9a-zA-Z-_]+)\/?$/i', $identity_url, $matches ) )
			$openid_name = $matches[2] . "deadjournal";
		else
			$openid_name = $identity_url;

		return array_merge( array( 'openid_name' => $openid_name, 'openid_identity' => $identity_url ), $data );
	}

	function create_identity_url( $type, $name )
	{
		if ( $type == 'lj' )
			$identity_url = 'http://www.livejournal.com/users/' . $name. '/';
		elseif ( $type == 'dj' )
			$identity_url = 'http://www.deadjournal.com/users/' . $name . '/';
		else
			$identity_url = $name;

		return $identity_url;
	}

    function parse_identity_url( $identity_url )
    {
        $url = normalize_url( $identity_url );
        
        $data = OpenIDConsumer::http_get( $url );
        if( !$data )
            return null;

        $links = OpenIDConsumer::get_server_and_delegate( $data );
        $server = $links['server'];
        $delegate = $links['delegate'];

        if( $server == '' )
            return null;

        if( $delegate != '' )
            $server_id = $delegate;
        else
            $server_id = $url;

        return array( 'identity_url' => $url, 'server_id' => $server_id, 'openid_server' => $server );
    }

    function http_get( $url )
    {
		$parts	= parse_url( $url );

		$params	= isset( $parts['query'] ) ? $parts['query'] : null;
    	
		if( is_array( $params ) )
			$params = http_build_query( $params );

		$curl = curl_init( $url . ( $params != '' ? '?' . $params : '' ) );

		curl_setopt( $curl, CURLOPT_FOLLOWLOCATION, true );
		curl_setopt( $curl, CURLOPT_HEADER, false );
		curl_setopt( $curl, CURLOPT_HTTPGET, true );
		curl_setopt( $curl, CURLOPT_POST, false );

		curl_setopt( $curl, CURLOPT_RETURNTRANSFER, true );

		$ret = curl_exec( $curl );
		curl_close( $curl );

		return $ret;
    }

	function http_post( $url, $params )
	{
		$curl = curl_init( $url );
		if( $curl )
		{
			curl_setopt( $curl, CURLOPT_POST, true );
			curl_setopt( $curl, CURLOPT_POSTFIELDS, $params );
			curl_setopt( $curl, CURLOPT_URL, $url );
			curl_setopt( $curl, CURLOPT_RETURNTRANSFER, true );

			$data = curl_exec( $curl );

			curl_close( $curl );
        }

		$data = trim( $data );
		$ret = array();
		$lines = explode( "\n", $data );
		foreach( $lines as $line )
		{
			$pair = explode( ':', $line, 2 );
			if( count( $pair ) != 2 )
				continue;
			
			list( $key, $value ) = $pair;
			$ret[trim( $key )] = trim( $value );
        }

		return $ret;
    }

    function get_server_and_delegate( $text )
    {
    	preg_match_all( '/(<[\s]*link[^>]*rel[\s]*=[\s]*"openid.server"[^>]*>)/', $text, $link_match );
    	preg_match_all( '/href[\s]*=[\s]*"([^"]*)"/', $link_match[1][0], $href_match );
    	$server = $href_match[1][0];

    	preg_match_all( '/(<[\s]*link[^>]*rel[\s]*=[\s]*"openid.delegate"[^>]*>)/', $text, $link_match );
    	preg_match_all( '/href[\s]*=[\s]*"([^"]*)"/', $link_match[1][0], $href_match );
    	$delegate = $href_match[1][0];

    	return array( 'server' => $server, 'delegate' => $delegate );
    }

    function get_association_by_server( $server_url )
    {
    	global $wpdb, $table_prefix;
    	$table_name = $table_prefix . 'openid_assoc_handles';
    	$assoc_info = $wpdb->get_row( 'SELECT * FROM '.$table_name.' WHERE server_url="'.$server_url.'"', ARRAY_A );
    	if( !$assoc_info )
    	{
    		$assoc_info = OpenIDConsumer::associate( $server_url );
			$wpdb->query( 'INSERT INTO '.$table_name.' (server_url, assoc_handle, secret, expires_at) VALUES ("'.$assoc_info['server_url'].'", "'.$assoc_info['assoc_handle'].'","' . $assoc_info['secret'] . '", "'.$assoc_info['expires_at'].'" )' );
    	}
    	else
    	{
    		if( time() > $assoc_info['expires_at'] )
    		{
    			$assoc_info = OpenIDConsumer::associate( $server_url );
    			$wpdb->query( 'UPDATE '.$table_name.' SET assoc_handle="'.$assoc_info['assoc_handle'].'", secret="'. $assoc_info['secret'] . '", expires_at="'.$assoc_info['expires_at'].'" WHERE server_url="'.$assoc_info['server_url'].'"' );
    		}
    	}

    	$assoc_info['secret'] = base64_decode( $assoc_info['secret'] );
    	return $assoc_info;
    }

    function get_association_by_handle( $handle )
    {
    	global $wpdb, $table_prefix;
    	$table_name = $table_prefix . 'openid_assoc_handles';
    	$assoc_info = $wpdb->get_row( 'SELECT * FROM '.$table_name.' WHERE assoc_handle="'.$handle.'"', ARRAY_A );
    	if( $assoc_info )
    	{
	    	$assoc_info['secret'] = base64_decode( $assoc_info['secret'] );
	    	return $assoc_info;
	    }
		else
			return null;
    }

    function associate( $server_url )
    {
        $query = http_build_query( array( 'openid.mode' => 'associate', 'openid.assoc_type' => 'HMAC-SHA1' ) );

        $results = OpenIDConsumer::http_post( $server_url, $query );

        $assoc_type = $results['assoc_type'];
        if( $assoc_type != 'HMAC-SHA1' )
        	return null;

        $assoc_handle = $results['assoc_handle'];
        $secret = $results['mac_key'];
        $expires_in = isset( $results['expires_in'] ) ? $results['expires_in'] : 0;
        $expires_at = time() + $expires_in;

        return array( 'server_url' => $server_url, 'assoc_handle' => $assoc_handle, 'secret' => $secret, 'expires_at' => $expires_at );
    }

	function remove_association( $assoc_handle )
	{
    	global $wpdb, $table_prefix;
    	$table_name = $table_prefix . 'openid_assoc_handles';
    	$wpdb->query( 'DELETE FROM ' . $table_name . ' WHERE assoc_handle="' . $assoc_handle . '"' );
	}

    function checkid_setup( $server_url, $server_id, $trust_root, $return_to, $assoc_handle )
    {
    	$params['openid.mode'] = 'checkid_setup';
		$params['openid.identity'] = $server_id;
    	$params['openid.trust_root'] = $trust_root;
    	$params['openid.return_to'] = $return_to;
    	$params['openid.assoc_handle'] = $assoc_handle;

    	$sep = strchr( $server_url, '?' ) ? '&': '?';
    	$location = $server_url . $sep . http_build_query( $params );
    	header( 'Location: ' . $location );
    	exit;
    }

    function checkid_immediate( $server_url, $server_id, $trust_root, $return_to, $assoc_handle )
    {
    	$params['openid.mode'] = 'checkid_immediate';
		$params['openid.identity'] = $server_id;
    	$params['openid.trust_root'] = $trust_root;
    	$params['openid.return_to'] = $return_to;
    	$params['openid.assoc_handle'] = $assoc_handle;

    	$sep = strchr( $server_url, '?' ) ? '&': '?';
    	$location = $server_url . $sep . http_build_query( $params );
    	header( 'Location: ' . $location );
    	exit;
    }

	function check_authentication( $query )
	{		
		$url_info = OpenIDConsumer::parse_identity_url( $query['openid_identity'] );
		if( $url_info['openid_server'] == '' )
			die( 'Couldn\'t find OpenID Server<br/>' );

    	$params['openid.mode'] = 'check_authentication';
		$params['openid.identity'] = $url_info['server_id'];
    	$params['openid.return_to'] = $query['openid_return_to'];
    	$params['openid.assoc_handle'] = $query['openid_assoc_handle'];
    	$params['openid.signed'] = $query['openid_signed'];
    	$params['openid.sig'] = $query['openid_sig'];

    	$response = OpenIDConsumer::http_post( $url_info['openid_server'], http_build_query( $params ) );

    	if( $response['is_valid'] == 'true' )
    		return true;
		else
			return false;
	}

    function delete_old_files()
    {
		if( $handle = opendir( './tmp/' ) )
		{
			/* This is the correct way to loop over the directory. */
			while( false !== ( $file = readdir( $handle ) ) )
			{
				if( filemtime( './tmp/' . $file ) + 24 * 3600 < time() )
					@unlink( './tmp/' . $file );
			}

			closedir( $handle );
		}
    }
}

class OpenIDServer
{
	function associate()
	{
    	global $wpdb, $table_prefix;
    	$table_name = $table_prefix . 'openid_associations';

    	do
    	{
    		$association = null;
			$handle = base64_encode( OpenIDServer::random_string( 20, 33, 126 ) );
    		$association = $wpdb->get_row( 'SELECT * FROM '.$table_name.' WHERE assoc_handle="' . $handle . '"', ARRAY_A );
		}
    	while( $association );

    	$secret = rand( 10000000, 99999999 );
    	$expires_in = 3600 * 2;
    	$expires_at = time() + $expires_in;

		$wpdb->query( 'INSERT INTO ' . $table_name . ' (assoc_handle, secret, expires_at) VALUES ("' . $handle . '", "' . $secret. '", "' . $expires_at . '" )' );

		// remove old associations
    	$wpdb->query( 'DELETE FROM ' . $table_name . ' WHERE expires_at < "' . time() . '"' );

		return array( 'assoc_handle' => $handle, 'secret' => $secret, 'expires_at' => $expires_at );
	}

	function get_association( $assoc_handle )
	{
    	global $wpdb, $table_prefix;
    	$table_name = $table_prefix . 'openid_associations';
		$association = $wpdb->get_row( 'SELECT * FROM '.$table_name.' WHERE assoc_handle="' . $assoc_handle . '"', ARRAY_A );
		return $association ? $association: null;
	}

	function remove_association( $assoc_handle )
	{
    	global $wpdb, $table_prefix;
    	$table_name = $table_prefix . 'openid_associations';
    	$wpdb->query( 'DELETE FROM ' . $table_name . ' WHERE assoc_handle="' . $assoc_handle . '"' );
	}

	function checkid_setup( $query, $openid_res )
	{
		if( !isset( $query['openid_trust_root'] ) )
			die( 'Error: no trust_root' );
		if( !isset( $query['openid_return_to'] ) )
			die( 'Error: no return_to' );
		if( !isset( $query['openid_identity'] ) )
			die( 'Error: no openid_identity' );

		foreach( $query as $key => $value )
		{
			if( $key != 'operation' && $key != 'trust_result' && $key != 'trust_once' )
				$params[$key] = $value;
		}

		OpenIDServer::is_identity_valid( $params );

		$reply = array( 'openid.return_to' => $params['openid_return_to'], 'openid.identity' => $params['openid_identity'] );

		if( isset( $params['openid_assoc_handle'] ) )
		{
			$assoc_info = OpenIDServer::get_association( $params['openid_assoc_handle'] );

			if( !$assoc_info || time() > $assoc_info['expires_at'] )
			{
				if( $assoc_info )
					OpenIDServer::remove_association( $assoc_info['assoc_handle'] );

				$assoc_info = OpenIDServer::associate();

				$reply['openid.invalidate_handle'] = $params['openid_assoc_handle'];
			}
		}
		else
			$assoc_info = OpenIDServer::associate();

		if( $assoc_info )
			$reply['openid.assoc_handle'] = $assoc_info['assoc_handle'];

		$reply['openid.mode'] = $openid_res;
		$_signed_fields = array('mode', 'identity', 'return_to');

		list( $signed, $sig ) = sign_reply( $reply, $assoc_info['secret'], $_signed_fields );

		$reply['openid.signed'] = $signed;
		$reply['openid.sig'] = $sig;

		$sep = strchr( $params['openid_return_to'], '?' ) ? '&': '?';
		$redirect = $params['openid_return_to'] . $sep . http_build_query( $reply );

		return $redirect;
	}

	function check_authentication( $query )
	{
		file_put_contents( './tmp/ddt.txt', 'ok' );
		$assoc_info = OpenIDServer::get_association( $query['openid_assoc_handle'] );

		if( $assoc_info && time() < $assoc_info['expires_at'] )
		{
            $token = $query;
            $token['openid_mode'] = 'id_res';

			$signed_fields = explode( ',', trim( $query['openid_signed'] ) );
			list( $signed, $sig ) = sign_reply( $token, $assoc_info['secret'], $signed_fields );

			if( $sig == $query['openid_sig'] )
				$reply['is_valid'] = 'true';
			else
				$reply['is_valid'] = 'false';
        }
		else
		{
			if( $assoc_info )
				OpenIDServer::remove_association( $query['openid_assoc_handle'] );

			$reply['is_valid'] = 'false';
			$reply['invalidate_handle'] = $query['openid_assoc_handle'];
        }

		$response = '';
		foreach( $reply as $key => $value )
			$response .= sprintf( "%s:%s\n", $key, $value );

		return $response;
	}

	function do_i_trust( $query )
	{
		if( isset( $query['trust'] ) && $query['trust'] == 'yes' )
			return true;

    	global $wpdb, $table_prefix;
    	$table_name = $table_prefix . 'openid_trust';
		$trust = $wpdb->get_row( 'SELECT * FROM ' . $table_name . ' WHERE identity="' . $query['openid_identity'] . '" AND trust_root="' . $query['openid_trust_root'] . '"', ARRAY_A );
		if( !$trust )
			return false;

		return true;
	}

	function trust_always( $query )
	{
		global $wpdb, $table_prefix;
    	$table_name = $table_prefix . 'openid_trust';
		$trust = $wpdb->get_row( 'SELECT * FROM ' . $table_name . ' WHERE identity="' . $query['openid_identity'] . '" AND trust_root="' . $query['openid_trust_root'] . '"', ARRAY_A );
		if( !$trust )
			$wpdb->query( 'INSERT INTO ' . $table_name . ' (identity, trust_root) VALUES ("' . $query['openid_identity'] . '", "'. $query['openid_trust_root'] . '" )' );
	}

	function is_identity_valid( $query )
	{
		if( !isset( $query['openid_identity'] ) )
			die( 'This page is used only for <a href="http://openid.net/">OpenID</a> validation.' );

		$identity = $query['openid_identity'];

		get_currentuserinfo();
		global $userdata;

		if( !$userdata || $userdata->user_login == '' )
		{
			echo 'You should login to verify <a href="'.get_settings('siteurl').'/wp-login.php">OpenID</a> identity.<br>';
			return false;
		}

		if( $userdata->user_level == 10 )
			return true;

		$parts = parse_url( $identity );
		$author_name = '';
		$author_id = 0;

		if( isset( $parts['path'] ) )
		{
			$tok = strtok( $parts['path'], '/' );
			while( $tok != false )
			{
				if( $tok == 'author' )
				{
					$tok = strtok( '/' );
					if( $tok )
						$author_name = $tok;
					break;
				}
			}
		}

		if( isset( $parts['query'] ) )
		{
			parse_str( $parts['query'], $query );
			if( isset( $query['author_name'] ) )
				$author_name = $query['author_name'];
			else if( isset( $query['author'] ) )
				$author_id = $query['author'];
		}

		if( $author_name != '' )
		{
			if( $author_name != $userdata->user_login )
				die( 'You cannot verify this OpenID identity as ' . $userdata->user_login . '.' );
		}
		else if( $author_id != 0 )
		{
			if( $author_id != $userdata->ID )
				die( 'You cannot verify this OpenID identity as ' . $userdata->user_login . '.' );
		}
		else
			die( 'You cannot verify this OpenID identity, try to use another one.' );
		
		return true;
	}

	function random_string( $length, $from = 0, $to = 255 )
	{
        $str = '';
        for( $i = 0; $i < $length; $i++ )
            $str .= chr( rand( $from, $to ) );

        return $str;
	}
}

/**
This class was taken from PHP-openid-0.0.8.3 library. 
http://videntity.org/openid/
**/
class oidUtilPhpSha1
{
	function hmacsha1( $key, $text )
	{
		if( function_exists( 'mhash' ) )
			return mhash( MHASH_SHA1, $text, $key );
		else
			return oidUtilPhpSha1::_hmacsha1( $key, $text );
    }
    
	function sha1( $s )
	{
		if( function_exists( 'mhash' ) )
            return mhash( MHASH_SHA1, $s );
        else if( function_exists( 'sha1' ) )
        {
			if( (int)PHP_VERSION>=5 )
				return sha1( $s, true );
			else
				return pack( "H*", sha1( $s ) );
        }
        else
			return oidUtilPhpSha1::_sha1_raw( $s );
    }

	// private
	// adapted from from http://php.net/manual/en/function.sha1.php#39492
	// mark at dot BANSPAM dot pronexus dot nl
	function _hmacsha1( $key,$data )
	{
		$blocksize=64;
		if( strlen( $key ) > $blocksize )
			$key=oidUtilPhpSha1::sha1( $key );

		$key = str_pad( $key, $blocksize, chr( 0x00 ) );
		$ipad = str_repeat( chr( 0x36 ), $blocksize );
		$opad = str_repeat( chr( 0x5c ), $blocksize );
		$hmac = oidUtilPhpSha1::sha1( ( $key^$opad ) . oidUtilPhpSha1::sha1( ( $key^$ipad ) . $data ) );
		return $hmac;
	}    
    
	// private
	// php implementation of sha1, for older php versions. reportedly fast.
	// adapted from http://php.net/manual/en/function.sha1.php#48856
	// mina86 at tlen dot pl
	function _sha1_step(&$H, $str)
	{
		$A = $H[0]; $B = $H[1]; $C = $H[2]; $D = $H[3]; $E = $H[4];
		$W = array_values( unpack( 'N16', $str ) );
		for( $i = 0; $i<16; ++$i )
			$W[$i+16] = &$W[$i];

		$t = 0;
		do //  0 <= t < 20
		{		
			$s = $t & 0xf;
			if( $t >= 16 )
				$W[$s] = oidUtilPhpSha1::_sha1_s( $W[$s+13] ^ $W[$s+8] ^ $W[$s+ 2] ^ $W[$s] );

			$TEMP = ($D ^ ($B & ($C ^ $D))) + 0x5A827999 + oidUtilPhpSha1::_sha1_s( $A, 5 ) + $E + $W[$s];
			$E = $D; $D = $C; $C = oidUtilPhpSha1::_sha1_s( $B, 30 ); $B = $A; $A = $TEMP;
		}
		while( ++$t < 20 );

		do // 20 <= t < 40
		{
			$s = $t & 0xf;
			$W[$s] = oidUtilPhpSha1::_sha1_s( $W[$s+13] ^ $W[$s+8] ^ $W[$s+ 2] ^ $W[$s] );

			$TEMP = ( $B ^ $C ^ $D ) + 0x6ED9EBA1 + oidUtilPhpSha1::_sha1_s( $A, 5 ) + $E + $W[$s];
			$E = $D; $D = $C; $C = oidUtilPhpSha1::_sha1_s( $B, 30 ); $B = $A; $A = $TEMP;
		}
		while( ++$t < 40 );

		do // 40 <= t < 60
		{
			$s = $t & 0xf;
			$W[$s] = oidUtilPhpSha1::_sha1_s( $W[$s+13] ^ $W[$s+8] ^ $W[$s+ 2] ^ $W[$s] );

			$TEMP = (($B & $C) | ($D & ($B | $C))) + 0x8F1BBCDC + oidUtilPhpSha1::_sha1_s( $A, 5 ) + $E + $W[$s];
			$E = $D; $D = $C; $C = oidUtilPhpSha1::_sha1_s($B, 30); $B = $A; $A = $TEMP;
		}
		while( ++$t < 60 );

		do // 60 <= t < 80
		{
			$s = $t & 0xf;
			$W[$s] = oidUtilPhpSha1::_sha1_s( $W[$s+13] ^ $W[$s+8] ^ $W[$s+ 2] ^ $W[$s] );

			$TEMP = ($B ^ $C ^ $D) + 0xCA62C1D6 + oidUtilPhpSha1::_sha1_s( $A, 5 ) + $E + $W[$s];
			$E = $D; $D = $C; $C = oidUtilPhpSha1::_sha1_s( $B, 30 ); $B = $A; $A = $TEMP;
		}
		while( ++$t < 80 );

		$H = array($H[0] + $A, $H[1] + $B, $H[2] + $C, $H[3] + $D, $H[4] + $E);
	}

	// private
	// php implementation of sha1, for older php versions. reportedly fast.
	// adapted from http://php.net/manual/en/function.sha1.php#48856
	// mina86 at tlen dot pl
	function _sha1_s( $X, $n = 1 )
	{
		return ( $X << $n ) | ( ( $X & 0x80000000 ) ? ( ( $X >> 1 ) & 0x07fffffff | 0x40000000 ) >> ( 31-$n ) : $X >> ( 32-$n ) );
	}

	// private
	// php implementation of sha1, for older php versions. reportedly fast.
	// adapted from http://php.net/manual/en/function.sha1.php#48856
	// mina86 at tlen dot pl
	function &_sha1_raw( $str )
	{
		$l = strlen( $str );
		$str = str_pad( "$str\x80\0\0\0\0", ( $l&~63 ) + ( ( ( $l&63 ) < 56 ) ? 60 : 124 ), "\0" ) . pack( 'N', $l << 3 );
		$l = strlen( $str );

		$H = array( 0x67452301, 0xEFCDAB89, 0x98BADCFE, 0x10325476, 0xC3D2E1F0 );
		for( $i = 0; $i<$l; $i += 64 )
			oidUtilPhpSha1::_sha1_step( $H, substr( $str, $i, 64 ) );

		$raw = pack( 'N*', $H[0], $H[1], $H[2], $H[3], $H[4] );
		return $raw;
	}
}
?>